/*

Copyright (c) 2000-2012 Samsung Electronics Co., Ltd All Rights Reserved 
 
This file is part of Smartsearch
Written by Junghyun Kim <jh1114.kim@samsung.com>
 
PROPRIETARY/CONFIDENTIAL
 
This software is the confidential and proprietary information of 
SAMSUNG ELECTRONICS (Confidential Information). You shall not 
disclose such Confidential Information and shall use it only in 
accordance with the terms of the license agreement you entered 
into with SAMSUNG ELECTRONICS.  

SAMSUNG make no representations or warranties about the suitability 
of the software, either express or implied, including but not limited
to the implied warranties of merchantability, fitness for a particular
purpose, or non-infringement. SAMSUNG shall not be liable for any
damages suffered by licensee as a result of using, modifying or 
distributing this software or its derivatives.
 
*/



#ifndef __DEF_PS_DEBUG_UTIL_H__
#define __DEF_PS_DEBUG_UTIL_H__

#ifdef __ENABLE_DEBUG_UTIL

#include <assert.h>

#ifdef __ENABLE_PLATFORM_DEBUG

#include <dlog.h>

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "SEARCH"

#define SEARCH_FUNC_START \
			do {\
				LOGD("[%s() START ----->]\n", __FUNCTION__);\
			}while(0)

#define SEARCH_FUNC_END \
			do {\
				LOGD("[%s() <----- END]\n", __FUNCTION__);\
			}while(0)

#define SEARCH_FUNC_LINE \
			do {\
				LOGD("[%s() \t%s:%d]\n", __FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
			}while(0)

#define SEARCH_DEBUG_LOG(fmt, arg...) \
			do {\
				LOGD("\x1b[32m[SEARCH][D][" fmt "]\x1b[0m\n", ##arg);\
			}while(0)

#define SEARCH_DEBUG_WARNING(fmt, arg...) \
			do {\
				LOGW("\x1b[33m[SEARCH][W][" fmt "]\x1b[0m\n", ##arg);\
			}while(0)

#define SEARCH_DEBUG_ERROR(fmt, arg...) \
			do {\
				LOGE("\x1b[31m[SEARCH][E][" fmt "]\x1b[0m\n", ##arg);\
			}while(0)

#define SEARCH_ASSERT_IF(cond) \
			do {\
				if(cond) {\
					LOGE("\x1b[31m[%s() \t%s:%d][ASSERT]\x1b[0m",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					assert(0);\
				}\
			}while(0)

#define SEARCH_RETV_IF(cond,ret) \
			do {\
				if(cond) {\
					LOGW("\x1b[33m[%s() \t%s:%d][RET_IF]\x1b[0m",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return ret;\
				}\
			}while(0)

#define SEARCH_RET_IF(cond) \
			do {\
				if(cond) {\
					LOGW("\x1b[33m[%s() \t%s:%d][RET_IF]\x1b[0m",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return;\
				}\
			}while(0)

#define SEARCH_RET_IF_STR_INVALID(str,ret) \
			do {\
				if((str == NULL) || strlen(str) == 0) {\
					LOGW("\x1b[33m[%s() \t%s:%d][RET_STR_IF]\x1b[0m",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return ret;\
				}\
			}while(0)

#else

#include <stdio.h>

#define SEARCH_FUNC_START \
			do {\
				fprintf(stderr, "[SEARCH][%s() START ----->]\n", __FUNCTION__);\
			}while(0)

#define SEARCH_FUNC_END \
			do {\
				fprintf("[SEARCH][%s() <----- END]\n", __FUNCTION__);\
			}while(0)

#define SEARCH_FUNC_LINE \
			do {\
				fprintf("[SEARCH][%s() \t%s:%d]\n", __FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
			}while(0)

#define SEARCH_DEBUG_LOG(fmt, arg...) \
			do {\
				fprintf(stderr, "[SEARCH][D][" fmt "\n", ##arg);\
			}while(0)

#define SEARCH_DEBUG_WARNING(fmt, arg...) \
			do {\
				fprintf(stderr, "[SEARCH][W][" fmt "\n", ##arg);\
			}while(0)

#define SEARCH_DEBUG_ERROR(fmt, arg...) \
			do {\
				fprintf(stderr, "[SEARCH][E][" fmt "\n", ##arg);\
			}while(0)

#define SEARCH_ASSERT_IF(cond) \
			do {\
				if(cond) {\
					fprintf(stderr, "[%s() \t%s:%d][ASSERT]",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					assert(0);\
				}\
			}while(0)

#define SEARCH_RETV_IF(cond,ret) \
			do {\
				if(cond) {\
					fprintf(stderr, "[%s() \t%s:%d][RET_IF]",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return ret;\
				}\
			}while(0)

#define SEARCH_RET_IF(cond) \
			do {\
				if(cond) {\
					fprintf(stderr, "[%s() \t%s:%d][RET_IF]",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return;\
				}\
			}while(0)

#define SEARCH_RET_IF_STR_INVALID(str,ret) \
			do {\
				if((str == NULL) || strlen(str) == 0) {\
					fprintf("\x1b[33m[%s() \t%s:%d][RET_STR_IF]\x1b[0m",__FUNCTION__, strrchr(__FILE__, '/')+1, __LINE__);\
					return ret;\
				}\
			}while(0)
			
#endif

#else

#define SEARCH_FUNC_START
#define SEARCH_FUNC_END
#define SEARCH_FUNC_LINE
#define SEARCH_DEBUG_LOG(fmt, arg...)
#define SEARCH_DEBUG_WARNING(fmt, arg...)
#define SEARCH_DEBUG_ERROR(fmt, arg...)
#define SEARCH_ASSERT_IF(cond)
#define SEARCH_RETV_IF(cond,ret)
#define SEARCH_RET_IF(cond)
#define SEARCH_RET_IF_STR_INVALID(str,ret)

#endif				// __ENABLE_DEBUG_UTIL

#endif				// __DEF_PS_DEBUG_UTIL_H__
