/*

Copyright (c) 2000-2012 Samsung Electronics Co., Ltd All Rights Reserved 
 
This file is part of Smartsearch
Written by Junghyun Kim <jh1114.kim@samsung.com>
 
PROPRIETARY/CONFIDENTIAL
 
This software is the confidential and proprietary information of 
SAMSUNG ELECTRONICS (Confidential Information). You shall not 
disclose such Confidential Information and shall use it only in 
accordance with the terms of the license agreement you entered 
into with SAMSUNG ELECTRONICS.  

SAMSUNG make no representations or warranties about the suitability 
of the software, either express or implied, including but not limited
to the implied warranties of merchantability, fitness for a particular
purpose, or non-infringement. SAMSUNG shall not be liable for any
damages suffered by licensee as a result of using, modifying or 
distributing this software or its derivatives.
 
*/



#include <string.h>
#include "smartsearch.h"
#include "Elementary.h"

static const char* g_search_category_icon[SEARCH_CATE_BTN_IMG_TYPE_MAX] = 
{
	"B10_Phone.png", 
	"B10_Phone_press.png",
};

void __search_searchbar_entry_changed_cb(void *data, Evas_Object * obj,
				       void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	const char *text;
	text = elm_entry_markup_to_utf8(elm_object_text_get(ad->search_entry));

	SEARCH_DEBUG_LOG("text : %s", text);

	if (elm_object_focus_get(ad->search_bar)) {
		if (elm_entry_is_empty(ad->search_entry))
			elm_object_signal_emit(ad->search_bar, "elm,state,eraser,hide", "elm");
		else
			elm_object_signal_emit(ad->search_bar, "elm,state,eraser,show", "elm");
	} else {
		if (elm_entry_is_empty(ad->search_entry)) 
			elm_object_signal_emit(ad->search_bar, "elm,state,guidetext,show", "elm");
		else
			elm_object_signal_emit(ad->search_bar, "elm,state,guidetext,hide", "elm");
	}

	if ((text != NULL) && (strlen(text) > 0)
		    && (SEARCH_STRCMP(text, ad->search_word) == 0)) {
			SEARCH_DEBUG_LOG("text is same as prev text");
	} else {
		if(ad->idler_search)
			ecore_idler_del(ad->idler_search);
		
		ad->idler_search = ecore_idler_add(search_set_result_list, ad);
	}		

	SEARCH_FUNC_END;
	return;
}


static void __search_searchbar_entry_focused_cb(void *data, Evas_Object *obj, void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	if (!elm_entry_is_empty(ad->search_entry))
		elm_object_signal_emit(ad->search_bar, "elm,state,eraser,show", "elm");

	elm_object_signal_emit(ad->search_bar, "elm,state,guidetext,hide", "elm");
	elm_object_signal_emit(ad->search_bar, "cancel,in", "");

	SEARCH_FUNC_END;
}

static void __search_searchbar_entry_unfocused_cb(void *data, Evas_Object *obj, void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	if (elm_entry_is_empty(ad->search_entry))
		elm_object_signal_emit(ad->search_bar, "elm,state,guidetext,show", "elm");

	elm_object_signal_emit(ad->search_bar, "elm,state,eraser,hide", "elm");

	SEARCH_FUNC_END;
}

static void __search_searchbar_bg_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	elm_object_focus_set(ad->search_entry, EINA_TRUE);

	SEARCH_FUNC_END;
}

static void __search_searchbar_eraser_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source) // When X marked button is clicked, empty entry's contents.
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	elm_entry_entry_set(ad->search_entry, "");

	SEARCH_FUNC_END;
}

static void __search_searchbar_key_down_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	Evas_Event_Key_Down *ev = (Evas_Event_Key_Down *) event_info;

	if (SEARCH_STRCMP(ev->key, "Return") == 0) {
                             ecore_imf_context_input_panel_hide
			    ((Ecore_IMF_Context *)elm_entry_imf_context_get
			     (ad->search_entry));
	}

	SEARCH_FUNC_END;
}

static void __search_searchbar_cancel_clicked_cb(void *data, Evas_Object * obj,
						 void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	const char* text;

	elm_object_focus_set(ad->cate_btn, EINA_TRUE);
	evas_object_hide(ad->search_cancel_btn);
	elm_object_signal_emit(ad->search_bar, "cancel,out", "");
	text = elm_entry_entry_get(ad->search_entry);

	if (text != NULL && strlen(text) > 0)
		elm_entry_entry_set(ad->search_entry, NULL);

	search_set_result_list(ad);

	malloc_trim(0);

	SEARCH_FUNC_END;
}

static void __search_searchbar_gl_drag_scroll_start_cb(void *data, Evas_Object * obj, void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	Ecore_IMF_Context *imf_context = (Ecore_IMF_Context *)elm_entry_imf_context_get(ad->search_entry);

	if (imf_context) {
		ecore_imf_context_input_panel_hide(imf_context);
	}

	SEARCH_FUNC_END;
}


static void
__search_searchbar_state_event_cb(void *data, Ecore_IMF_Context *imf_context, int value)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	switch (value)
	{
		case ECORE_IMF_INPUT_PANEL_STATE_SHOW:
			evas_object_smart_callback_add(
					ad->search_gl, 
					"scroll,drag,start", 
					__search_searchbar_gl_drag_scroll_start_cb, 
					data);				
		break;
		case ECORE_IMF_INPUT_PANEL_STATE_HIDE:
		// ISE state has changed to ISE_STATE_HIDE status
			evas_object_smart_callback_del(
					ad->search_gl, 
					"scroll,drag,start", 
					__search_searchbar_gl_drag_scroll_start_cb);
		break;
	}

	SEARCH_FUNC_END;	
}

static void __search_searchbar_create(void *data)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	if (!ad)
		return;

	if (ad->search_bar) {
		elm_object_signal_callback_del(
								ad->search_bar,
								"elm,bg,clicked",
								"elm",
								__search_searchbar_bg_clicked_cb);
		elm_object_signal_callback_del(
								ad->search_bar,
								"elm,eraser,clicked",
								"elm",
								__search_searchbar_eraser_clicked_cb);

		evas_object_del(ad->search_bar);
		ad->search_bar = NULL;
	}

	if (ad->search_entry) {
		evas_object_smart_callback_del(
								ad->search_entry,
								"changed",
								__search_searchbar_entry_changed_cb);
		evas_object_smart_callback_del(
								ad->search_entry,
								"preedit,changed",
								__search_searchbar_entry_changed_cb);
		evas_object_smart_callback_del(
								ad->search_entry,
								"focused",
								__search_searchbar_entry_focused_cb);
		evas_object_smart_callback_del(
								ad->search_entry,
								"unfocused",
								__search_searchbar_entry_unfocused_cb);

		evas_object_del(ad->search_entry);
		ad->search_entry = NULL;
	}

	if (ad->search_cancel_btn) {
		evas_object_smart_callback_del(
								ad->search_cancel_btn,
								"clicked",
					       __search_searchbar_cancel_clicked_cb);
		evas_object_del(ad->search_cancel_btn);
		ad->search_cancel_btn = NULL;
	}

	ad->search_bar = elm_layout_add(ad->navi_bar);
	elm_layout_theme_set(ad->search_bar , "layout", "searchbar", "cancel_button");

	ad->search_cancel_btn = elm_button_add(ad->search_bar);
	elm_object_part_content_set(ad->search_bar, "button_cancel", ad->search_cancel_btn);
	elm_object_style_set(ad->search_cancel_btn, "searchbar/default");
	elm_object_text_set(ad->search_cancel_btn, dgettext("sys_string", "IDS_COM_SK_CANCEL"));
	
	ad->search_entry = elm_entry_add(ad->search_bar);
	elm_entry_scrollable_set(ad->search_entry, EINA_TRUE);
	elm_entry_single_line_set(ad->search_entry, EINA_TRUE);
	elm_object_part_content_set(ad->search_bar, "elm.swallow.content", ad->search_entry);
	elm_object_part_text_set(ad->search_bar, "elm.guidetext", dgettext("sys_string", "IDS_COM_BODY_SEARCH"));
	elm_entry_cnp_mode_set(ad->search_entry, ELM_CNP_MODE_PLAINTEXT);

	evas_object_size_hint_weight_set(ad->search_bar, EVAS_HINT_EXPAND, 0);
	evas_object_size_hint_align_set(ad->search_bar, EVAS_HINT_FILL, 0.0);

	elm_entry_input_panel_layout_set(ad->search_entry, ELM_INPUT_PANEL_LAYOUT_NORMAL);
	Ecore_IMF_Context *imf_context = (Ecore_IMF_Context *)elm_entry_imf_context_get(ad->search_entry);
	ecore_imf_context_input_panel_event_callback_add(imf_context,
							 ECORE_IMF_INPUT_PANEL_STATE_EVENT,
							 __search_searchbar_state_event_cb,
							 data);

	/* Set Searchbar Callback */
	elm_object_signal_callback_add(
								ad->search_bar,
								"elm,bg,clicked",
								"elm",
								__search_searchbar_bg_clicked_cb,
								ad);
	elm_object_signal_callback_add(
								ad->search_bar,
								"elm,eraser,clicked",
								"elm",
								__search_searchbar_eraser_clicked_cb,
								ad);
	
	/* Set Search-Entry Callback */
	evas_object_smart_callback_add(
								ad->search_entry,
								"changed",
								__search_searchbar_entry_changed_cb,
								ad);
	evas_object_smart_callback_add(
								ad->search_entry,
								"preedit,changed",
								__search_searchbar_entry_changed_cb,
								ad);

	evas_object_smart_callback_add(
								ad->search_entry,
								"focused",
								__search_searchbar_entry_focused_cb,
								ad);
	evas_object_smart_callback_add(
								ad->search_entry,
								"unfocused",
								__search_searchbar_entry_unfocused_cb,
								ad);
	evas_object_event_callback_add(
								ad->search_entry,
								EVAS_CALLBACK_KEY_DOWN, 
								__search_searchbar_key_down_cb,
				       			ad);

	/* Set Searchbar Cancel Button Callback */
	evas_object_smart_callback_add(
								ad->search_cancel_btn,
								"clicked",
								__search_searchbar_cancel_clicked_cb,
								ad);

	evas_object_show(ad->search_bar);

	SEARCH_FUNC_END;
}

static Evas_Object *__search_init_noresult_view(void *data)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	Evas_Object *noc;

	noc = elm_layout_add(ad->navi_bar);
	elm_layout_theme_set(noc, "layout", "nocontents", "search");
	elm_object_part_text_set(noc, "elm.text", dgettext("sys_string", "IDS_COM_BODY_NO_SEARCH_RESULTS"));
	elm_object_focus_allow_set(noc, EINA_TRUE);

	SEARCH_FUNC_END;

	return noc;
}

Evas_Object *search_searchbar_category_icon_add(int cate_type, int press_type, Evas_Object *parent)
{
	SEARCH_FUNC_START;

	Evas_Object* cate_icon;
	char buf[128] = { 0, };

	cate_icon = elm_icon_add(parent);
	
	snprintf(buf, sizeof(buf), SEARCH_ICON_PATH "%s", g_search_category_icon[press_type]);

	elm_icon_file_set(cate_icon, buf, NULL);

	evas_object_size_hint_aspect_set(cate_icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
	elm_icon_resizable_set(cate_icon, EINA_TRUE, EINA_TRUE);

	SEARCH_FUNC_END;

	return cate_icon;
}

void search_searchbar_cb(void *data, Evas_Object * obj, void *event_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	
	Evas_Object *cate_btn;
	Evas_Object *cate_icon;
	int category = 0;
	int err = 0;

	/* create layout */

	ad->sb_layout = elm_layout_add(ad->navi_bar);
	elm_layout_file_set(ad->sb_layout,
						SEARCH_EDJ,
						"smartsearch/search_result");

	category = SEARCH_TYPE_PHONE;

	cate_icon = search_searchbar_category_icon_add(category, 
													SEARCH_CATE_BTN_IMG_TYPE_UNPRESSED, 
													ad->sb_layout);

	cate_btn = elm_button_add(ad->sb_layout);
	elm_object_content_set(ad->cate_btn, cate_icon);
	
	evas_object_show(cate_btn);
	ad->cate_btn = cate_btn;

	elm_object_part_content_set(ad->sb_layout, "searchbar_area_category", cate_btn);
	
	/* set search bar area */
	__search_searchbar_create(ad);
	elm_object_part_content_set(ad->sb_layout, "searchbar_area_entry", ad->search_bar);

	/* set search result area */
	ad->noresult_view = __search_init_noresult_view(ad);

	elm_object_part_content_set(ad->sb_layout, 
						"list_noresult",
						ad->noresult_view);

	evas_object_size_hint_weight_set(
							ad->sb_layout, 
							EVAS_HINT_EXPAND,
					 		EVAS_HINT_EXPAND);

	SEARCH_FUNC_END;
}
